package edu.umd.dawn.common.utils;

import edu.umd.dawn.common.exceptions.BaseExceptions;
import edu.umd.dawn.common.exceptions.DawnException;
import org.apache.commons.lang3.builder.EqualsBuilder;
import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.apache.commons.lang3.builder.ToStringBuilder;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;

public class OffsetLimitPageRequest implements Pageable {

    private int limit;
    private long offset;
    private final Sort sort;

    /**
     * Creates a new {@link OffsetLimitPageRequest} with sort parameters applied.
     *
     * @param offset zero-based offset.
     * @param limit  the size of the elements to be returned.
     * @param sort   can be {@literal null}.
     */
    public OffsetLimitPageRequest(long offset, int limit, Sort sort) {
        if (offset < 0) {
            throw new DawnException(BaseExceptions.INVALID_OFFSET((int) offset));
        }

        if (limit < 1) {
            throw new DawnException(BaseExceptions.INVALID_LIMIT(limit));
        }
        this.limit = limit;
        this.offset = offset;
        this.sort = sort;
    }

    /**
     * Creates a new {@link OffsetLimitPageRequest} with sort parameters applied.
     *
     * @param offset zero-based offset.
     * @param limit  the size of the elements to be returned.
     */
    public OffsetLimitPageRequest(long offset, int limit) {
        this(offset, limit, Sort.unsorted());
    }

    public static OffsetLimitPageRequest of(long offset, int limit) {
        return new OffsetLimitPageRequest(offset, limit);
    }

    public static OffsetLimitPageRequest of(long offset, int limit, Sort sort) {
        return new OffsetLimitPageRequest(offset, limit, sort);
    }

    @Override
    public int getPageNumber() {
        return (int) offset / limit;
    }

    @Override
    public int getPageSize() {
        return limit;
    }

    @Override
    public long getOffset() {
        return offset;
    }

    @Override
    public Sort getSort() {
        return sort;
    }

    @Override
    public Pageable next() {
        return new OffsetLimitPageRequest(getOffset() + getPageSize(), getPageSize(), getSort());
    }

    public OffsetLimitPageRequest previous() {
        return hasPrevious() ? new OffsetLimitPageRequest(getOffset() - getPageSize(), getPageSize(), getSort()) : this;
    }

    @Override
    public Pageable previousOrFirst() {
        return hasPrevious() ? previous() : first();
    }

    @Override
    public Pageable first() {
        return new OffsetLimitPageRequest(0, getPageSize(), getSort());
    }

    @Override
    public boolean hasPrevious() {
        return offset > limit;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;

        if (!(o instanceof OffsetLimitPageRequest)) return false;

        OffsetLimitPageRequest that = (OffsetLimitPageRequest) o;

        return new EqualsBuilder()
                .append(limit, that.limit)
                .append(offset, that.offset)
                .append(sort, that.sort)
                .isEquals();
    }

    @Override
    public int hashCode() {
        return new HashCodeBuilder(17, 37)
                .append(limit)
                .append(offset)
                .append(sort)
                .toHashCode();
    }

    @Override
    public String toString() {
        return new ToStringBuilder(this)
                .append("limit", limit)
                .append("offset", offset)
                .append("sort", sort)
                .toString();
    }

    @Override
    public Pageable withPage(int pageNumber) {
        return null;
    }
}
